
MenuTitleR = 0
MenuTitleG = 102
MenuTitleB = 102
MenuTitleA = 150

MenuButtonR = 0
MenuButtonG = 102
MenuButtonB = 102
MenuButtonA = 130

MenuActiveButtonR = 0
MenuActiveButtonG = 255
MenuActiveButtonB = 255
MenuActiveButtonA = 150

ChaoticaGUI = {}
ChaoticaGUI.menu = {}
ChaoticaGUI.buttons = {}
ChaoticaGUI.navButtons = 	{
				["menuOpen"] = 
				{
					["keys"] = {},
					["menu"] = ""
				}, 
				["menuClose"] = {["keys"] = {}},
				["menuPrev"] = {["keys"] = {}},
				["navUp"] = {["keys"] = {}},
				["navDown"] = {["keys"] = {}},
				["select"] = {["keys"] = {}},
				["navBack"] = {["keys"] = {}}
			}
ChaoticaGUI.activeButton = 1
ChaoticaGUI.activeMenu = ""
ChaoticaGUI.statusText = {["text"] = "", ["time"] = 0}

ChaoticaGUI.titleColor = {["r"] = MenuTitleR, ["g"] = MenuTitleG, ["b"] = MenuTitleB, ["a"] = MenuTitleA}
ChaoticaGUI.buttonColor = {["r"] = MenuButtonR, ["g"] = MenuButtonG, ["b"] = MenuButtonB, ["a"] = MenuButtonA}
ChaoticaGUI.activeButtonColor = {["r"] = MenuActiveButtonR, ["g"] = MenuActiveButtonG, ["b"] = MenuActiveButtonB, ["a"] = MenuActiveButtonA}


function ChaoticaGUI.createMenu(menuName, prevMenu, title, xpos, ypos, xscale, yscale, buttonSpacing, textScale, font, saveButtonPos)
	if(ChaoticaGUI.menu[menuName] == nil) then
		ChaoticaGUI.menu[menuName] = {}
		lastOffset = ypos + yscale + buttonSpacing
	end
	
	ChaoticaGUI.menu[menuName] = {
		["prevMenu"] = prevMenu,
		["title"] = title,
		["xpos"] = xpos,
		["ypos"] = ypos,
		["xscale"] = xscale,
		["yscale"] = yscale,
		["spacing"] = buttonSpacing,
		["lastOffset"] = lastOffset,
		["numButtons"] = 0,
		["textScale"] = textScale,
		["lastButton"] = 1,
		["saveButtonPos"] = saveButtonPos,
		["font"] = font
	}
end

function ChaoticaGUI.addButton(parentMenu, text, funct, args, toggleable, textScale, font)
	if(ChaoticaGUI.buttons[parentMenu] == nil or ChaoticaGUI.buttons[parentMenu]["settings"] == nil) then
		ChaoticaGUI.buttons[parentMenu] = {["settings"] = {}}
	end

	local currButtonNum = #ChaoticaGUI.buttons[parentMenu].settings + 1
	ChaoticaGUI.menu[parentMenu].numButtons = currButtonNum

	if(toggleable) then
		toggledOn = false
	else
	    toggledOn = nil
	end

	ChaoticaGUI.buttons[parentMenu].settings[currButtonNum] = {
		["parentMenu"] = parentMenu,
		["text"] = text,
		["funct"] = funct,
		["args"] = args,
		["font"] = font,
		["xpos"] = ChaoticaGUI.menu[parentMenu].xpos,
		["ypos"] = ChaoticaGUI.menu[parentMenu].lastOffset,
		["xscale"] = ChaoticaGUI.menu[parentMenu].xscale,
		["yscale"] = ChaoticaGUI.menu[parentMenu].yscale,
		["textScale"] = textScale,
		["toggleable"] = toggleable,
		["toggledOn"] = toggledOn
	}

	ChaoticaGUI.menu[parentMenu]["lastOffset"] = (ChaoticaGUI.menu[parentMenu].lastOffset + ChaoticaGUI.menu[parentMenu].yscale + ChaoticaGUI.menu[parentMenu].spacing)
end

function ChaoticaGUI.setActiveMenu(menuName)
	ChaoticaGUI.activeMenu = menuName
	if(ChaoticaGUI.menu[ChaoticaGUI.activeMenu] ~= nil) then
		if(ChaoticaGUI.menu[ChaoticaGUI.activeMenu].saveButtonPos or ChaoticaGUI.menu[ChaoticaGUI.activeMenu].saveButtonPos == nil) then
			ChaoticaGUI.activeButton = ChaoticaGUI.menu[ChaoticaGUI.activeMenu].lastButton
		else
		    ChaoticaGUI.activeButton = 1
		end
	end
end

function ChaoticaGUI.updateStatusText(text, time)
	ChaoticaGUI.statusText = {["text"] = text, ["time"] = time}
end

function ChaoticaGUI.isActiveButtonToggledOn()
	local activeButton = ChaoticaGUI.buttons[ChaoticaGUI.activeMenu].settings[ChaoticaGUI.activeButton]
	if(activeButton.toggleable) then
		return activeButton.toggledOn
	elseif(activeButton.toggleable == false) then
	    return nil
	end
end

function ChaoticaGUI.isButtonToggledOn(menu, index)
	local activeButton = ChaoticaGUI.buttons[menu].settings[index]
	if(activeButton.toggleable) then
		return activeButton.toggledOn
	elseif(activeButton.toggleable == false) then
	    return nil
	end
end

function ChaoticaGUI.toggleActiveButtonState()
	local activeButton = ChaoticaGUI.buttons[ChaoticaGUI.activeMenu].settings[ChaoticaGUI.activeButton]


	if(ChaoticaGUI.isActiveButtonToggledOn() and activeButton.toggleable) then
		activeButton.toggledOn = false
	elseif(ChaoticaGUI.isActiveButtonToggledOn() == false and activeButton.toggleable) then
		activeButton.toggledOn = true
	end
end

function ChaoticaGUI.toggleButtonState(menu, index)
	local activeButton = ChaoticaGUI.buttons[menu].settings[index]
	
	if(ChaoticaGUI.isButtonToggledOn(menu, index) and activeButton.toggleable) then
		activeButton.toggledOn = false
	elseif(ChaoticaGUI.isButtonToggledOn(menu, index) == false and activeButton.toggleable) then
		activeButton.toggledOn = true
	end
end

function ChaoticaGUI.removeButton(parentMenu, index)
	ChaoticaGUI.buttons[parentMenu].settings[index] = nil
	ChaoticaGUI.menu[parentMenu]["lastOffset"] = (ChaoticaGUI.menu[parentMenu].lastOffset - ChaoticaGUI.menu[parentMenu].yscale - ChaoticaGUI.menu[parentMenu].spacing)
end

function ChaoticaGUI.removeMenu(menuName)
	ChaoticaGUI.menu[menuName] = nil
end

function ChaoticaGUI.drawStatusText()
	if(nextDrawTime == nil or prevText ~= ChaoticaGUI.statusText.text) then
		nextDrawTime = natives.GAMEPLAY.GET_GAME_TIMER() + ChaoticaGUI.statusText.time
	end

	if(nextDrawTime > natives.GAMEPLAY.GET_GAME_TIMER()) then
		prevText = ChaoticaGUI.statusText.text
		natives.UI.SET_TEXT_FONT(0)
		natives.UI.SET_TEXT_SCALE(0.55, 0.55)
		natives.UI.SET_TEXT_COLOUR(255, 255, 255, 255)
		natives.UI.SET_TEXT_WRAP(0.0, 1.0)
		natives.UI.SET_TEXT_OUTLINE(1)
		natives.UI.SET_TEXT_CENTRE(true)
		natives.UI.SET_TEXT_DROPSHADOW(1, 0, 0, 255, 255)
		natives.UI.SET_TEXT_EDGE(1, 0, 0, 0, 205)
		natives.UI._SET_TEXT_ENTRY("STRING")
		natives.UI._ADD_TEXT_COMPONENT_STRING(ChaoticaGUI.statusText.text)
		natives.UI._DRAW_TEXT(0.5, 0.9)
	else
		ChaoticaGUI.statusText = {["text"] = "", ["time"] = 0}
		nextDrawTime = nil
	end
end

function ChaoticaGUI.drawChaoticaGUI()
	if(ChaoticaGUI.activeMenu ~= "" and ChaoticaGUI.activeMenu ~= nil) then
		local name = ChaoticaGUI.activeMenu

		if(ChaoticaGUI.menu[name] ~= nil) then
			local xpos = ChaoticaGUI.menu[name].xpos
			local ypos = ChaoticaGUI.menu[name].ypos
			local xscale = ChaoticaGUI.menu[name].xscale
			local yscale = ChaoticaGUI.menu[name].yscale
			
			local text = ChaoticaGUI.menu[name].title
			local textScale = ChaoticaGUI.menu[name].textScale
			local font = ChaoticaGUI.menu[name].font
			
			local colors = ChaoticaGUI.titleColor

			ChaoticaGUI.addButtonText(text, xpos, ypos - 0.007, textScale, font)
			natives.GRAPHICS.REQUEST_STREAMED_TEXTURE_DICT("helicopterhud", true)
			natives.GRAPHICS.DRAW_SPRITE("helicopterhud", "hud_outline", xpos, ypos, xscale, yscale, 0, colors.r, colors.g, colors.b, 255)

			if(ChaoticaGUI.buttons[name] ~= nil) then
				for i, v in pairs(ChaoticaGUI.buttons[name].settings) do
					local buttonSettings = ChaoticaGUI.buttons[name].settings[i]
					local xpos = buttonSettings.xpos
					local ypos = buttonSettings.ypos
					local xscale = buttonSettings.xscale
					local yscale = buttonSettings.yscale
					
					local font = buttonSettings.font
					local text = buttonSettings.text
					local textScale = buttonSettings.textScale
					local colors = ChaoticaGUI.buttonColor

					if(buttonSettings.toggledOn == true) then
						text = buttonSettings.text .. " [ON]"
					elseif(buttonSettings.toggledOn == false) then
						text = buttonSettings.text .. "~c~ [OFF]"
					end
					
					if(i == ChaoticaGUI.activeButton) then
						colors = ChaoticaGUI.activeButtonColor
					end

					ChaoticaGUI.addButtonText(text, xpos, ypos, textScale, font)
					natives.GRAPHICS.REQUEST_STREAMED_TEXTURE_DICT("helicopterhud", true)
					natives.GRAPHICS.DRAW_SPRITE("helicopterhud", "hud_lock", xpos, ypos, xscale, yscale, 0, colors.r, colors.g, colors.b, 255)
				end
			else
				print("Menu ", name, " has no buttons!")
				ChaoticaGUI.setActiveMenu("")
			end
		else
			print("No menu created with the name: ", ChaoticaGUI.activeMenu)
			ChaoticaGUI.setActiveMenu("")
		end
	end
end

function ChaoticaGUI.addButtonText(text, xpos, ypos, textScale, font)
	if(text ~= nil and xpos ~= nil and ypos ~= nil and textScale ~= nil and font ~= nil) then
		natives.UI.SET_TEXT_FONT(font)
		natives.UI.SET_TEXT_SCALE(0.0, textScale)
		natives.UI.SET_TEXT_COLOUR(255, 255, 255, 255)
		natives.UI.SET_TEXT_CENTRE(true)
		natives.UI.SET_TEXT_OUTLINE(1)
		natives.UI.SET_TEXT_DROPSHADOW(1, 0, 0, 255, 255)
		natives.UI.SET_TEXT_EDGE(0, 0, 0, 0, 0)
		natives.UI._SET_TEXT_ENTRY("STRING")
		natives.UI._ADD_TEXT_COMPONENT_STRING(text)
		natives.UI._DRAW_TEXT(xpos, ypos - 0.0125)
	else
		print("text: ", text, ", xpos: ", xpos, ", ypos: ", ypos, ", textScale: ", textScale, ", font: ", font)
	    print("There seems to be a nil value when trying to add text to button (see above)")
	end
end

function ChaoticaGUI.runButtonFunct(currButton)
	if(type(currButton.funct) == "function" and currButton.args ~= nil and type(currButton.args) ~= "string" and type(currButton.args) ~= "number" and type(currButton.args) ~= "boolean") then
		currButton.funct(currButton.args[1], currButton.args[2], currButton.args[3], currButton.args[4], currButton.args[5], currButton.args[6], currButton.args[7], currButton.args[8], currButton.args[9], currButton.args[10], currButton.args[11], currButton.args[12], currButton.args[13], currButton.args[14], currButton.args[15], currButton.args[16], currButton.args[17], currButton.args[18], currButton.args[19], currButton.args[20])
		return true
	end

	if(type(currButton.funct) == "function" and type(currButton.args) == "string" or type(currButton.args) == "number" or type(currButton.args) == "boolean") then
		currButton.funct(currButton.args)
		return true
	end

	if(currButton.funct ~= nil and currButton.args == nil  ) then
		currButton.funct()
		return true
	end

	if(currButton.funct == nil) then
		print("Unabled to run the function")
		return false
	end
end

function ChaoticaGUI.tick()
	ChaoticaGUI.drawStatusText()
	ChaoticaGUI.drawChaoticaGUI()

	for i, key in pairs(ChaoticaGUI.navButtons.menuOpen.keys) do
		if(IsKeyDown(key)) then
			natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "QUIT_WHOOSH", "HUD_MINI_GAME_SOUNDSET", false)
			if(ChaoticaGUI.activeMenu == ChaoticaGUI.navButtons.menuOpen.menu) then
				ChaoticaGUI.setActiveMenu(ChaoticaGUI.menu[ChaoticaGUI.activeMenu].prevMenu)
			else
			    ChaoticaGUI.setActiveMenu(ChaoticaGUI.navButtons.menuOpen.menu)
			    ChaoticaGUI.currentSelection = 1
			end
		end
	end
	if ChaoticaGUI.activeMenu ~= "" then
		for i, key in pairs(ChaoticaGUI.navButtons.menuClose.keys) do
			if(IsKeyDown(key)) then
				natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false)
				ChaoticaGUI.setActiveMenu("")
			end
		end
	end
	if ChaoticaGUI.activeMenu ~= "" then
		for k, key in pairs(ChaoticaGUI.navButtons.navUp.keys) do
			if(IsKeyDown(key) and ChaoticaGUI.activeMenu ~= "") then
				ChaoticaGUI.activeButton = ChaoticaGUI.activeButton - 1
				ChaoticaGUI.menu[ChaoticaGUI.activeMenu].lastButton = ChaoticaGUI.activeButton
				natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false)
				
				if(ChaoticaGUI.activeButton <= 0) then
					ChaoticaGUI.activeButton = ChaoticaGUI.menu[ChaoticaGUI.activeMenu].numButtons
					ChaoticaGUI.menu[ChaoticaGUI.activeMenu].lastButton = ChaoticaGUI.activeButton
				end
			end
		end
	end
	if ChaoticaGUI.activeMenu ~= "" then
		for k, key in pairs(ChaoticaGUI.navButtons.navDown.keys) do
			if(IsKeyDown(key) and ChaoticaGUI.activeMenu ~= "") then
				ChaoticaGUI.activeButton = ChaoticaGUI.activeButton + 1
				ChaoticaGUI.menu[ChaoticaGUI.activeMenu].lastButton = ChaoticaGUI.activeButton
				natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false)

				if(ChaoticaGUI.activeButton > ChaoticaGUI.menu[ChaoticaGUI.activeMenu].numButtons) then
					ChaoticaGUI.activeButton = 1
					ChaoticaGUI.menu[ChaoticaGUI.activeMenu].lastButton = ChaoticaGUI.activeButton
				end
			end
		end
	end
	if ChaoticaGUI.activeMenu ~= "" then
		for k, key in pairs(ChaoticaGUI.navButtons.select.keys) do
			if(IsKeyDown(key) and ChaoticaGUI.activeMenu ~= "") then
				
				local currButton = ChaoticaGUI.buttons[ChaoticaGUI.activeMenu].settings[ChaoticaGUI.activeButton]
				natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false)

				if(ChaoticaGUI.runButtonFunct(currButton)) then
					if(currButton.toggleable) then
						ChaoticaGUI.toggleActiveButtonState()
					end
				end
			end
		end
	end
	if ChaoticaGUI.activeMenu ~= "" then
		for k, key in pairs(ChaoticaGUI.navButtons.navBack.keys) do
			if(IsKeyDown(key) and ChaoticaGUI.activeMenu ~= "") then
				ChaoticaGUI.setActiveMenu(ChaoticaGUI.menu[ChaoticaGUI.activeMenu].prevMenu)
				natives.AUDIO.PLAY_SOUND_FRONTEND(-1, "Pin_Centred", "DLC_HEIST_BIOLAB_PREP_HACKING_SOUNDS", false);
			end
		end
	end
end

function ChaoticaGUI:OnError()
	--self:Kill()
	--addon.Load("RENAMEME")
end